#!/bin/sh
# <BEGIN COPYRIGHT>
# This file is part of the "Bad Penguin" GNU/Linux distribution
# Copyright (C) 1997-98-99 by Antonio GALLO
# Home Page at ......: http://www.badpenguin.org
# Home Page at ......: http://www.linux.it/~agx/
# e-mail contact ....: mailto:agx@linux.it	mailto:agx@geocities.com
# This program is distributed under the GNU General Public License
# You are not allowed to remove the copyright notice
# <END COPYRIGHT>


# This is based on /sbin/newuser by Mike Jackson <mhjack@tscnet.com>
# i (agx@usa.net) adjusted it to work with dialog in AGX-Linux
# This script is part of "Bad Penguin" - http://www.linux.it/~agx/
# 1.0.0 by agx@linux.it 
# 1999-04-27, Version 1.0.0 by AGX


# "useradd"
#	-c comment
#	-d home_dir
#	-e expire_date
#	-f inactive_time (disable the account if password was expired)
#	-g initial_group
#	-G group1, group2, 
#	-m -m skeleton -s shell
#	-u UID loginname

# "usermod" is the same as useradd
#	also it accepts -l login_name to change the account name

# "passwd" [-f|-s] [name]
# 	-f change the finger information
# "passwd" [-g] [-r|R] group
#	-g (change the group password)
#	-r (remove the password from the group)
#	-R (restrict the named group for all the users)
# "passwd" [-x max] [-n min] [-w warn] [-i inact] name
#	-x max password age
#	-n minimum age between two password changes
#	-w day of warning before password expire
#	-i if password expire disable the account after those n days	
# "passwd" {-l|-u|-d} name
# 	-l (lock) -u (unlock)


TEMPDIR="$HOME/tmp"
[ ! -d $TEMPDIR ] && mkdir -p $TEMPDIR
TEMPDATA=$TEMPDIR/tempdata.$$
trap 'rm -f $TEMPDATA ; exit $USCITA' 0



SystemGroups() {
  IFSOLD=$IFS
  IFS=":"
  while read x y ; do
    echo -n "$x "
  done < /etc/group
  IFS=$IFSOLD
}

GetExpireDay() {
	SOURCE=`expr $1 + 1`
	YEAR=1969
	REMAIN=0
	while [ $SOURCE -gt 0 ]; do
		YEAR=`expr $YEAR + 1`
		BISSESTILE=`expr $YEAR % 4`
		if [ $BISSESTILE -eq 0 ]; then
			BISSESTILE=1
			SOURCE=`expr $SOURCE - 366`
		else
			BISSESTILE=0
			SOURCE=`expr $SOURCE - 365`
		fi
	done
	#
	if [ $BISSESTILE -eq 0 ]; then
		REMAIN=`expr 365 + $SOURCE`
	else
		REMAIN=`expr 366 + $SOURCE`
	fi
	#
	MESE=0
	DAY=0
	while [ $REMAIN -gt 0 ]; do
		DAY=$REMAIN
		MESE=`expr $MESE + 1`
		case $MESE in
			11|4|6|9)
				GIORNI=30
				;;
			2)
				if [ $BISSESTILE -eq 1 ]; then
					GIORNI=29
				else
					GIORNI=28
				fi
				;;
			*)
				GIORNI=31
				;;
		esac
		REMAIN=`expr $REMAIN - $GIORNI`
	done
	#
	[ "$MESE" = "0" ] && MESE=1
	[ "$DAY" = "0" ] && DAY=1
	echo "$MESE/$DAY/$YEAR"
}


GetEtcPassword() {
  IFSOLD=$IFS
  IFS=":"
  while read tmp_user tmp_pass tmp_uid tmp_gid tmp_comment tmp_home tmp_shell; do
    if [ "$tmp_user" = "$1" ]; then
      S_HOME="$tmp_home"
      S_SHELL="$tmp_shell"
      # Fake finger informations
      IFS=","
      echo $tmp_comment | read S_COMMENT
      IFS=$IFSOLD
    fi
  done < /etc/passwd
  IFS=$IFSOLD
}


GetEtcShadow() {
  IFSOLD=$IFS
  IFS=":"
  while read tmp_user tmp_pass tmp_last tmp_may tmp_must tmp_warn tmp_expire tmp_disable tmp_reserved; do
    if [ "$tmp_user" = "$1" ]; then
      #S_EXPIRE is returned in day since 1/1/1970
      if [ "$tmp_disable" = "" ]; then
        S_EXPIRE=""
      else
        S_EXPIRE="`GetExpireDay $tmp_disable`"
      fi

      #Inactivity age
      if [ "$tmp_expire" = "" ]; then
        S_INACTIVEAGE="-1"
      else
        S_INACTIVEAGE="$tmp_expire"
      fi

      #Password age
      S_PASSAGE="$tmp_must"
      #Password MinAge
      S_PASSMINAGE="$tmp_may"
      #Password Warning
      S_PASSWARN="$tmp_warn"

      IFS=$IFSOLD
    fi
  done < /etc/shadow
  IFS=$IFSOLD
}



# =====   B E G I N   =====

# Ensure that root is running this script.
if [ $UID -ne 0 ]; then
    dialog --msgbox "You must be root to add news users!" 8 50
    USCITA=1 ; exit 1
fi



#////////////////
# Default Values
#////////////////

# --- useradd/usermod ---
S_LOGIN="guest"			# @ User Name
S_COMMENT="guest account"	# @ Comment field into /etc/passwd
S_HOME="/home/$S_LOGIN"		# @ Home directory location (/home/username)
S_EXPIRE=""			# @ Account expiration date
S_INACTIVEAGE=-1		# @ Disable the account if password expire
S_IGROUP="users"		# @ Initial groups
S_OGROUP=""			# @ Other groups
C_SKELETON="/etc/skel"		# @ Constant Skeleton Directory
S_SHELL="/bin/bash"		# @ Default Shell (full path)
S_UID=""									# @ User UID
# --- passwd ---
S_PASSAGE=60			# @ Password age (0=Never Expire)
S_PASSMINAGE=0			#   minimum age between password change
S_PASSWARN=5			#   Day of warning before password epire
# --- misc ---

# ------------------------------------------------

if [ "$1" = "MODIFY" ]; then
  if [ "$2" = "" ]; then
    echo "Sorry, no account name to modify was specified !"
    USCITA=2 ; exit 2
  fi

  clear
  echoc GREEN "Loading account informations for $2 ..."

  # Running in modify mode not add-user mode
  ADD_MODE=1
  TITLE="Modify properties for the user $2"
  OPERATION="Post the account changes"

  # ... add here modified user values ...
  S_LOGIN=$2
  S_UID="`id $S_LOGIN -u`"
  S_IGROUP="`id $S_LOGIN -ng`"
  S_OGROUP="`id $S_LOGIN -nG`"    
  #TODO: Ricavare Expiration Data
  S_EXPIRE=""

  #TODO: Get pass warning
  S_PASSWARN=5

  # Scan for FullName, HomeDir and Default Shell
  GetEtcPassword $S_LOGIN

  # Scan for (expiration date), inactivity age
  GetEtcShadow $S_LOGIN

else
    ADD_MODE=0
    TITLE="Adding a new user"
    OPERATION="Add the new account and exit"
fi


# //////////////////////  B E G I N  /////////////////////
M_LOGIN=$S_LOGIN
M_UID=$S_UID
M_COMMENT=$S_COMMENT
M_IGROUP=$S_IGROUP
M_OGROUP=$S_OGROUP
M_HOME=$S_HOME
M_SHELL=$S_SHELL
M_EXPIRE=$S_EXPIRE
M_INACTIVEAGE=$S_INACTIVEAGE
#
M_PASSAGE=$S_PASSAGE
M_PASSMINAGE=$S_PASSMINAGE
M_PASSWARN=$S_PASSWARN


while [ 0 ]; do
  dialog --title "$TITLE" --menu "Change the default values than press OK" 25 79 15 \
    "Username"         ": $M_LOGIN" \
    "User ID"          ": $M_UID" \
    "Description"      ": $M_COMMENT" \
    "Group"            ": $M_IGROUP" \
    "More groups"      ": $M_OGROUP" \
    "Home dir"         ": $M_HOME" \
    "Shell"            ": $M_SHELL" \
    "Expiration Date"  ": $M_EXPIRE" \
    "Password Age"     ": $M_PASSAGE" \
    "Inactivity Age"   ": $M_INACTIVEAGE" \
    " " " " \
    "SAVE"     		"$OPERATION" \
    2> $TEMPDATA
  if [ $? = 0 ]; then
    choice="`cat $TEMPDATA`"
  else
    break
  fi
    
  case "$choice" in
    
  "Username")
    dialog --inputbox "Insert the user login account name" 8 50 2> $TEMPDATA
    if [ $? = 0 ]; then
      M_LOGIN="`cat $TEMPDATA`"
      if [ -z "$M_LOGIN" ]; then
        dialog --msgbox "Come on, man, you can't leave the login field empty..." 5 60
        M_LOGIN="$S_LOGIN"
      else
        M_HOME="/home/$M_LOGIN"
      fi
    fi
    ;;

	
  "User ID")
    dialog --inputbox "User ID (Suggestion: leaving this field blank automatically get the next ID available)" 9 50 2> $TEMPDATA
    if [ $? = 0 ]; then
      M_UID="`cat $TEMPDATA`"
    fi
    [ -z "$ID" ] && M_UID="$S_UID"
    ;;


  "Description")
    dialog --inputbox "Enter the account description (maybe the Full user name)" 9 60 2> $TEMPDATA
    if [ $? = 0 ]; then
      M_COMMENT="`cat $TEMPDATA`"
      [ -z "$M_COMMENT" ] && M_COMMENT="$S_COMMENT"
    fi
    ;;


  "Group")
    dialog --inputbox "Insert the name of a group for your new user.\nGroups are : `SystemGroups` " 10 60 2> $TEMPDATA
    if [ $? = 0 ]; then
      M_IGROUP="`cat $TEMPDATA`"
    fi
    [ -z "$M_IGROUP" ] && M_IGROUP="$S_IGROUP"
    ;;


  "More groups")
    dialog --inputbox "Additional groups that your user is part (type a comma separated list).\nGroups are : `SystemGroups`" 10 75 2> $TEMPDATA
    if [ $? = 0 ]; then
      M_OGROUP="`cat $TEMPDATA`"
      [ -z "$M_OGROUP" ] && M_OGROUP="$S_OGROUP"
    fi
    ;;


  "Home dir")
    dialog --inputbox "Full path of the user's home directory, usually under /home/" 8 50 2> $TEMPDATA
    if [ $? = 0 ]; then
      M_HOME="`cat $TEMPDATA`"
      [ -z "$M_HOME" ] && M_HOME="$S_HOME"
    fi
    ;;


  "Shell")
    dialog --inputbox "User login Shell (Suggestion: if this is a e-mail only account use /bin/false)" 9 50 2> $TEMPDATA
    if [ $? = 0 ]; then
      M_SHELL="`cat $TEMPDATA`"
      [ -z "$M_SHELL" ] && M_SHELL="$S_SHELL"
    fi
    ;;


  "Expiration Date")
    dialog --inputbox "User account expiration date. The date is specified in the format MM/DD/YY. (Suggestion: Leave blank if unsure)" 10 50 "$M_EXPIRE" 2> $TEMPDATA
    if [ $? = 0 ]; then
      M_EXPIRE="`cat $TEMPDATA`"				
      [ -z "$M_EXPIRE" ] && M_EXPIRE="$S_EXPIRE"
    fi
    ;;


  "Password Age")
    dialog --inputbox "Maximum number of day after which a password expire and the user should change it\n\
( Suggestions: '0'=password never expire )" \
	12 75 2> $TEMPDATA
    if [ $? = 0 ]; then
      M_PASSAGE="`cat $TEMPDATA`"				
      [ -z "$M_PASSAGE" ] && M_PASSAGE="$S_PASSAGE"
    fi
    ;;


  "Inactivity Age")
    dialog --inputbox "If a pasword expire specify the number of days to wait before to disable the account.\n\
( Suggestions: '0'=disable the account as soon the password has expired; \n'-1' disable this feature )" \
	12 75 2> $TEMPDATA
    if [ $? = 0 ]; then
      M_INACTIVEAGE="`cat $TEMPDATA`"				
      [ -z "$M_INACTIVEAGE" ] && M_INACTIVEAGE="$S_INACTIVEAGE"
    fi
    ;;
					
  "SAVE"|"MODIFY")
    # Calcola UID
    if [ -z "$M_UID" ]; then
      GUID=""
    else
      GUID="-u$M_UID"
    fi
    # Calcola gruppo iniziale
    GGID="-g$M_IGROUP"
    # Calcola gruppi
    if [ -z "$M_OGROUP" ]; then 
      GAGID=""
    else
      GAGID="-G$M_OGROUP"
    fi
    # Calcola home dir
    GHME="-d$M_HOME"
    # Calcola shell
    GSHL="-s$M_SHELL"
    # Calcola expiration date
    GEXP="-e$M_EXPIRE"
    [ -z "$M_EXPIRE" ] && GEXP=""

    # Calcola inactive age
    GINACTIVE="-f$M_INACTIVEAGE"
    [ -z "$M_INACTIVEAGE" ] && GINACTIVE=""
    
    # Perform Operation
    clear
    if [ $ADD_MODE = 0 ]; then
      echo "Adding a new user ..."
      useradd -c"$M_COMMENT" $GHME $GINACTIVE -m -k$C_SKELETON $GEXP $GGID $GAGID $GSHL $GUID $M_LOGIN
      REPLY=$?
      if [ $REPLY = 0 ]; then
	if [ -d $M_HOME ]; then
	  chmod 711 $M_HOME
	fi
	
	if [ ! -f /var/spool/mail/$M_LOGIN ]; then
	  echo "Adding Mailbox"
	  > /var/spool/mail/$M_LOGIN
	  chmod 660 /var/spool/mail/$M_LOGIN
	  chown $M_LOGIN.mail /var/spool/mail/$M_LOGIN
	fi
	
    	echo
	echo "Preparing user information for 'finger' ..."
	chfn $LOGIN
	echo
        echo "Preparing password ..."
        passwd -x $M_PASSAGE -n $M_PASSMINAGE -w $M_PASSWARN $M_LOGIN >/dev/null 2>&1
	REPLY=$?
	if [ $REPLY -ne 0 ]; then
          echoc RED "Error $REPLY while adding the new user password !" ; read x
          USCITA=1 ; exit 1
        fi
	passwd $M_LOGIN
	echo "Done."
      else
        echoc RED "Error $REPLY while adding the new user account !" ; read x
        USCITA=1 ; exit 1
      fi
    else
      # === Controlla valori modificati ===
      # Controllo Login
      if [ "$M_LOGIN" = "$S_LOGIN" ]; then
        M_LOGIN=""
      else
        M_LOGIN=" -l$M_LOGIN"
      fi		    
      # Controllo UID
      [ "$M_UID" = "$S_UID" ] && GUID=""
      # Controllo Initial Group
      [ "$M_IGROUP" = "$S_IGROUP" ] && GGID=""
			# Controllo Grupppi
      [ "$M_OGROUP" = "$S_OGROUP" ] && GAGID=""
      # Controllo Home
      if [ "$M_HOME" = "$S_HOME" ]; then 
        GHME=""
      else
        GHME="$GHME -m"
      fi
      # Controllo Shell
      [ "$M_SHELL" = "$S_SHELL" ] && GSHL=""
      # Controllo Expiration Date
      [ "$M_EXPIRE" = "$S_EXPIRE" ] && GEXP=""
      # Controllo Inactivity Age
      [ "$M_INACTIVEAGE" = "$S_INACTIVEAGE" ] && GINACTIVE=""
      #
      #
      echo "Modifying the user account ..."
      usermod -c"$M_COMMENT" $GHME $GINACTIVE $GEXP $GGID $GAGID $GSHL $GUID $M_LOGIN $S_LOGIN
      REPLY=$?
      if [ $REPLY -ne 0 ]; then
        echoc RED "Error $REPLY while changeing the user account !" ; read x
        USCITA=1 ; exit 1
      fi
      echo "Modifying the user password ..."
      passwd -x $M_PASSAGE -n $M_PASSMINAGE -w $M_PASSWARN $S_LOGIN >/dev/null 2>&1
      REPLY=$?
      if [ $REPLY -ne 0 ]; then
        echoc RED "Error $REPLY while changeing the user account !" ; read x
        USCITA=1 ; exit 1
      fi
      echo "done."
    fi
    echo
    echo "Please, press ENTER to continue ..."
    read x
    break
    ;;
	
    *)
      break
      ;;
  esac

done
USCITA=0
exit 0
