#!/bin/sh
# <BEGIN COPYRIGHT>
# This file is part of the "Bad Penguin" GNU/Linux distribution
# Copyright (C) 1997-98-99 by Antonio GALLO
# Home Page at ......: http://www.badpenguin.org
# Home Page at ......: http://www.linux.it/~agx/
# e-mail contact ....: mailto:agx@linux.it	mailto:agx@geocities.com
# This program is distributed under the GNU General Public License
# You are not allowed to remove the copyright notice
# <END COPYRIGHT>


# Manage a user choosed by the user_group.sh
# This script is part of "Bad Penguin" - http://www.linux.it/~agx/
# 1.0.0 by agx@linux.it 
# 1999-04-27, Version 1.0.0 by AGX

# Format of the password file, i.e. /etc/passwd
# A non-shadowed /etc/passwd file has the following format:
#	username:passwd:UID:GID:full_name:directory:shell
#
#	username	- The user (login) name
#	passwd		- The encoded password
#	UID		- Numerical user ID
#	GID		- Numerical default group ID
#	full_name	- The user's full name
#	directory	- User's home directory (Full pathname)
#	shell		- User's login shell (Full Pathname)
#
# For example:
# 	username:Npge08pfz4wuk:503:100:Full Name:/home/username:/bin/sh


# Format of the shadow file
# The /etc/shadow file contains the following information:
#	username:passwd:last:may:must:warn:expire:disable:reserved
# Where:
#	username 	- The User Name
#	passwd		- The Encoded password
#	last		- Days since Jan 1, 1970 that password was last changed
#	may		- Days before password may be changed
#	must		- Days after which password must be changed
#	warn		- Days before password is to expire that user is warned
#	expire		- Days after password expires that account is disabled
#	disable		- Days since Jan 1, 1970 that account is disabled
#	reserved	- A reserved field
#
# The previous example might then be:
#	username:Npge08pfz4wuk:9479:0:10000::::

[$TEMPDIR="$HOME/tmp"
[ ! -d $TEMPDIR ] && mkdir -p $TEMPDIR
TEMPSCRIPT=$TEMPDIR/tempscript.$$
TEMPDATA=$TEMPDIR/tempdata.$$
trap 'rm -f $TEMPSCRIPT $TEMPDATA ; exit $USCITA' 0
USCITA=0


while [ 0 ]; do
    clear
    echoc GREEN "Reading accounts informations ..."
    
    SECONDS_SINCE="`date +%s`"
    DAY_SINCE=$[ SECONDS_SINCE / 86400 ]
    IFS=":"
    SHADOW_USERNAME="#"

    rm -f $TEMPSCRIPT
    while [ ! "$SHADOW_USERNAME" = "" ]; do
	SHADOW_USERNAME=""
	read SHADOW_USERNAME SHADOW_PASSWORD SHADOW_LAST SHADOW_MAY SHADOW_MUST SHADOW_WARN SHADOW_EXPIRE SHADOW_DISABLE SHADOW_RESERVED

	case $SHADOW_PASSWORD in
		!*)
			SHADOW_EXPIRE="( * locked * )"
                	;;
		"*"*)
			SHADOW_EXPIRE="(  disabled  )"
			;;
		*)
			if [ "$SHADOW_DISABLE" = "" ]; then
				if [ "$SHADOW_EXPIRE" = "" ]; then
					SHADOW_EXPIRE="(never expire)"
				else
					SHADOW_EXPIRE="Password expire $SHADOW_EXPIRE"
				fi	
			else
				SHADOW_EXPIRE="Disabled in $[ SHADOW_DISABLE - DAY_SINCE ] days"
			fi			

			;;
	esac

	#------
	IFSOLD=$IFS
	IFS=":"
	S_COMMENT=""
	while read tmp_user tmp_pass tmp_uid tmp_gid tmp_comment tmp_home tmp_shell ; do
		if [ "$tmp_user" = "$SHADOW_USERNAME" ]; then
			#echo $tmp_user
			#echo $tmp_comment
			S_COMMENT="` echo $tmp_comment | cut -d, -f1 `" 
			break
		fi
	done < /etc/passwd
	IFS=$IFSOLD
	#------

	if [ ! "$SHADOW_USERNAME" = "" ]; then
        	echo -n "$SHADOW_USERNAME:$SHADOW_EXPIRE - $S_COMMENT:" >> $TEMPSCRIPT
	fi
    done < /etc/shadow
    
    # Display account
    dialog --title "AGX_Linux User Manager (@1998-99 by Antonio Gallo)" --menu "Choose a user to see his properties and then decide to access \n\
other releated options" 25 79 15 `cat $TEMPSCRIPT` 2> $TEMPSCRIPT

    if [ $? = 0 ]; then
	ACCOUNT="`cat $TEMPSCRIPT`"
dialog --title "Parameters Account: $ACCOUNT" --menu "Here the user $ACCOUNT parameters. Select an operation or CANCEL to exit.\n\
\n" 25 79 15 \
    "PASSWORD"	"Change the password for this user         " \
    "LOCK"   	"Lock this account                         " \
    "UNLOCK" 	"Unlock this account                       " \
    " "      	" " \
    "MODIFY" 	"Modify user account properties            " \
    "INFO" 	"Change the user's personal informations   " \
    "SHELL"     "Change the user's login shell             " \
    " "      	" " \
    "LASTLOG"   "Show the last logins for this user        " \
    "FAILLOG"   "Show the failure logs for this user       " \
    " "      	" " \
    "DELETE" 	"Remove this account                       " \
    "ERASE"  	"Remove this account and its home directory" 2> $TEMPDATA
    
	if [ $? = 0 ]; then
		CHOICE="`cat $TEMPDATA`"
		case "$CHOICE" in
		    PASSWORD)
			clear
			passwd $ACCOUNT
		    ;;
		    LOCK)
			clear
			echoc YELLOW "Locking the account $ACCOUNT"
			passwd -l $ACCOUNT
			echo ; echoc GREEN "done. Press ENTER to continue ..." ; read x
		    ;;
		    UNLOCK)
			clear
			echoc YELLOW "Unlocking the account $ACCOUNT"		    
			passwd -u $ACCOUNT
			echo ; echoc GREEN "done. Press ENTER to continue ..." ; read x
		    ;;
		    MODIFY)
			./user_new.sh MODIFY $ACCOUNT
		    ;;
		    DELETE)
			clear
			echo
			echoc YELLOW "Deleting account $ACCOUNT"
			userdel $ACCOUNT
			echo ; echoc GREEN "done. Press ENTER to continue ..." ; read x
		    ;;
		    ERASE)
			clear
			echo		    
			echoc YELLOW "Erasing account $ACCOUNT"		    
			userdel -r $ACCOUNT
			echo ; echoc GREEN "done. Press ENTER to continue ..." ; read x
		    ;;
                    INFO)
                        clear
                        chfn $ACCOUNT
			echo ; echoc GREEN "done. Press ENTER to continue ..." ; read x
			;;
                    SHELL)
                        clear
                        chsh $ACCOUNT
			echo ; echoc GREEN "done. Press ENTER to continue ..." ; read x
			;;
                    LASTLOG)
                        clear ; echo ; echo
                        lastlog -u $ACCOUNT
			echo ; echoc GREEN "Press ENTER to continue ..." ; read x
			;;
                    FAILLOG)
                        clear ; echo ; echo
                        faillog -p -u $ACCOUNT
			echo ; echoc GREEN "Press ENTER to continue ..." ; read x
			;;

		esac
	fi
	
    else
      break
    fi
    
done
